////////////////////////////////////////////////////////////////////////
//
// 46-927 Data Structures, Algorithms and AI
//
// File: AbstractOption.h
//
// Author:        Jeff Stephenson        
// Date:          September 17, 1998  
// Comments:      Declaration of an abstract Option class
//                See AbstractOption.cpp
//
//                This class is abstract.  Therefore, you must
//                derive a class from this class and add code
//                for the following pure virtual functions:
//
//                virtual double F(double s, double x);
//                virtual double Payoff(double s, double x);
//
//                The following member functions will be inherited
//                by the derived class:
//
//                void Calc_European_Option();
//                void GenPDAG();
//                void PrintPDAG();
//
//                The GenPDAG() function should be called before
//                Calc_European_Option or PrintPDAG()
//
//                Chapter 2 and Chapter 3 of the course notes
//                describe pricing an option on a PDAG.  This an 
//                implementation of the process they describe.
////////////////////////////////////////////////////////////////////////


#ifndef ABSTRACTOPT_H
#define ABSTRACTOPT_H

#include "LinkedList.h"

#define TESTDEBUG false

class CAbstractOption  
{
public:
	CAbstractOption(int T, double r, double u, double d, double s_0);
	virtual ~CAbstractOption();

    //Pure Virtual Function
    //How does the auxillary process evolve
    //x is the old value of the auxillary process
    //s is the new stock price
    virtual double F(double s, double x) = 0;

    //Pure Virtual Function
    //Payoff the option we are computing
    //in terms of the stock price and the
    //auxilliary process
    //t is the time value of the node we are
    //computing the payoff for
    virtual double Payoff(double s, double x, int t) = 0;

    //Calculate the Option using the European method
    void Calc_European_Option();

    //Generate the PDAG
    void GenPDAG();

    //Print the PDAG
    void PrintPDAG();

    //Fills pathArray with a random path
    void MonteCarloPath(CNode** pathArray);

    //Generates paths using MonteCarloPath and prints them
    void PrintPaths(int numPaths);

    //Returns the number of nodes in the lattice
    int CountLatticeNodes();

    double GetFinalOptionValue();



protected:

    //time horizon
    int m_timeHorizon;

    //short rate
    double m_rate;

    //up factor and down factor
    double m_upFactor;
    double m_downFactor;

    //initial stock price 
    double m_initial;

    //risk-nuetral probability
    //m_probUp is the probability of up move
    //m_probDown is the probability of down move
    double m_probUp;
    double m_probDown;

    //nodes[i] points to link list of
    //nodes with time i
    CLinkedList* m_nodes;

};

#endif 